/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.settings;

import java.util.Objects;

import de.ozgcloud.user.common.errorhandling.AccessForbiddenException;
import de.ozgcloud.user.common.errorhandling.FunctionalException;
import jakarta.inject.Inject;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.PATCH;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.MediaType;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.microprofile.config.inject.ConfigProperty;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.jboss.resteasy.reactive.ResponseStatus;
import org.jboss.resteasy.reactive.common.util.RestMediaType;

import de.ozgcloud.user.User;
import de.ozgcloud.user.UserService;
import io.quarkus.hal.HalEntityWrapper;

@Path(UserSettingsResource.USER_PATH)
public class UserSettingsResource {

	public static final String USER_PATH = "/api/user"; // NOSONAR
	public static final String SETTINGS_LINK_PATTERN = USER_PATH + "/{id}/settings";

	@Inject
	UserSettingsService userSettingsService;

	@Inject
	UserService userService;

	@Inject
	JsonWebToken jwt;

	@Inject
	UserSettingsResourceAssembler resourceAssembler;

	@ConfigProperty(name = "ozgcloud.user-manager.url")
	String userManagerUrl;

	@GET
	@Path("/{id}/settings")
	@Produces({ MediaType.APPLICATION_JSON, RestMediaType.APPLICATION_HAL_JSON })
	public HalEntityWrapper getUserSettings(@PathParam("id") String userId) {
		checkUserAccess(userId);

		return resourceAssembler.toResource(userSettingsService.getByUserId(userId), userId, userManagerUrl);
	}

	@PATCH
	@ResponseStatus(200)
	@Path("/{id}/settings")
	@Produces({ MediaType.APPLICATION_JSON, RestMediaType.APPLICATION_HAL_JSON })
	public HalEntityWrapper updateUserSettings(@PathParam("id") String userId, UserSettings userSettings) {
		checkUserAccess(userId);

		if (Objects.isNull(userSettings)) {
			throw new FunctionalException(() -> "Request Body missing.");
		}

		return userSettingsService.updateByUserId(userSettings, userId)
				.map(updatedSettings -> resourceAssembler.toResource(updatedSettings, userId, userManagerUrl))
				.orElseThrow(() -> new FunctionalException(() -> "Invalid user id"));
	}

	void checkUserAccess(String userId) {
		var user = userService.findByExternalId(jwt.getSubject());

		if (Objects.isNull(user) || hasNoAccess(user, userId)) {
			throw new AccessForbiddenException(UserSettings.class, userId);
		}
	}

	private boolean hasNoAccess(User user, String userId) {
		return !StringUtils.equals(user.getId().toHexString(), userId);
	}
}